//DevMpf.cc
 
/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPF is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/
/*
        Current Author: Marty Kraimer
        Original Author: Jim Kowalkowski (for Hideos)
        Date: 01JUL98
*/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <epicsMutex.h>
#include <epicsEvent.h>
#include <errlog.h>
#include <dbDefs.h>
#include <link.h>
#include <dbAccess.h>
#include <recSup.h>
#include <dbScan.h>
#include <alarm.h>
#include <recGbl.h>
 
#include "Message.h"
#include "ConnectMessage.h"

#include "DevMpf.h"

int DevMpfDebug = 0;


void DevMpf::bind()
{
    status = pMessageClient->bind(serverName,serverLocation);
    if(status) {
        errlogPrintf("MPF::bind error. server %s Record %s\n",
            serverName,precord->name);
        precord->pact = true;
        return;
    }
}

typedef long (*processCall)(dbCommon *);
extern "C" {
static void callDeviceSupport(CALLBACK* cback);
}

static void callDeviceSupport(CALLBACK* cback)
{
    dbCommon* pc;
    void *dummy;
    if(!interruptAccept) {
        printf("DevPMT: callDeviceSupport called when !interruptAccept\n");
        return;
    }
    callbackGetUser(dummy,cback);
    pc = (dbCommon *)dummy;
    rset *prset=(struct rset *)(pc->rset);
    processCall pcall = (processCall)(prset->process);
    /* process the record */
    dbScanLock(pc);
    (*pcall)(pc);
    dbScanUnlock(pc);
}

DevMpf::DevMpf(dbCommon* pr, DBLINK* pl,bool iointvalid)
: ioscanpvt(0), iointValid(iointvalid),
  status(0), precord(pr), plink(pl), userParm(0), pMessageClient(0),
  connectState(connectNo), replyMessage(0), state(waitStateNone)
{
    char name[100];
    int i;
    lockId = epicsMutexMustCreate();
    sendWaitSem = epicsEventMustCreate(epicsEventEmpty);
    if(plink->type!=VME_IO) {
	errlogPrintf("%s INP is not a vme link\n",precord->name);
	precord->pact = true;
	return;
    }
    pMessageClient = new MessageClient(DevMpf::messageClientCallback,
	(void *)precord);
    vmeio *pvmeio=(vmeio*)&(plink->value);
    // first field of parm is always the server name
    for(i=0;
       pvmeio->parm[i] && pvmeio->parm[i]!=',' && pvmeio->parm[i]!=' ';
       i++) name[i]=pvmeio->parm[i];
    if(pvmeio->parm[i]) userParm = &pvmeio->parm[i+1];
    name[i]='\0';
    serverName = (char *)calloc(i,sizeof(char *));
    strcpy(serverName,name);
    serverLocation = pvmeio->card;
    callbackSetCallback(callDeviceSupport,&cb);	
    callbackSetPriority(precord->prio,&cb);
    callbackSetUser(precord,&cb);
    if(iointvalid)  scanIoInit(&ioscanpvt);
    precord->dpvt=this;
    return;
}

DevMpf::~DevMpf() {
    epicsEventDestroy(sendWaitSem);
    epicsMutexDestroy(lockId);
    free(serverName);
    // What about pMessageClient ???
}

long DevMpf::linconv(void* v,int pass)
{
    dbCommon* precord=(dbCommon*)v;
    DevMpf* pdevMpf=(DevMpf*)precord->dpvt;
    if(pdevMpf==0) return(0);
    pdevMpf->convert(precord,pass);
    return(0);
}

void DevMpf::connectIO(dbCommon*,Message *message)
{
    delete message;
    if(connectState==connectYes) {
        // Note that messageClientCallback already called lock
        if(state==waitStateConnect) {
            state=waitStateNone;
            sendWatchdog.cancel();
            epicsEventSignal(sendWaitSem);
        }
        return;
    }
    // on disconnect force record processing
    if(interruptAccept) callbackRequest(&cb);
}

void DevMpf::receiveReply(dbCommon *,Message *message)
{
    printf("%s DevMpf::receiveReply called\n",precord->name);
    delete message;
}

void DevMpf::outOfBandIO(dbCommon *,Message *message)
{
    printf("%s DevMpf::outOfBandIO called\n",precord->name);
    delete message;
}

long DevMpf::convert(dbCommon*,int) { return(0); }

bool DevMpf::connectWait(double timeout)
{
    lock();
    if(connectState==connectYes) {
        unlock();
        return(true);
    }
    state = waitStateConnect;
    sendWatchdog.start(timeout,sendWaitCallback,this);
    unlock();
    epicsEventMustWait(sendWaitSem);
    state = waitStateNone;
    return(connectState==connectYes);
}

int DevMpf::send(Message *pmessage,replyType type)
{
    pmessage->setClientType((int32)type);
    status = pMessageClient->send(pmessage);
    if(status==0) {
        if(type==replyTypeCompleteIO) precord->pact = TRUE;
    } else {
	delete pmessage;
	recGblSetSevr(precord,COMM_ALARM,INVALID_ALARM);
    }
    return(status);
}

void DevMpf::sendWaitCallback(void *pvt)
{
    DevMpf *pDevMpf = (DevMpf*)pvt;
    pDevMpf->lock();
    if(pDevMpf->state!=waitStateNone) {
        pDevMpf->state = waitStateTimeout;
        epicsEventSignal(pDevMpf->sendWaitSem);
    }
    pDevMpf->unlock();
}

int DevMpf::sendWait(Message *pmessage,replyType type,double timeout)
{
    if(connectState != connectYes) {
        lock();
        state = waitStateConnect;
        sendWatchdog.start(timeout,sendWaitCallback,this);
        unlock();
        epicsEventMustWait(sendWaitSem);
        if(state==waitStateTimeout) {
            state = waitStateNone;
	    delete pmessage;
            return(-1);
        }
    }
    if(type==replyTypeNone) return(send(pmessage,type));
    lock();
    state = waitStateReply;
    sendWatchdog.start(timeout,sendWaitCallback,this);
    unlock();
    status = send(pmessage,type);
    if(status) {
        lock();
        sendWatchdog.cancel();
        state = waitStateNone;
        unlock();
        return(status);
    }
    epicsEventMustWait(sendWaitSem);
    if(state==waitStateTimeout) {
        state = waitStateNone;
        return(-1);
    }
    return(0);
}

long DevMpf::ioint(int , dbCommon* pcommon,IOSCANPVT* iopvt)
{
    DevMpf* pdevMpf=(DevMpf*)pcommon->dpvt;
    if(!pdevMpf->iointValid) return(0);
    if(pdevMpf==0) return(0);
    *iopvt=pdevMpf->ioscanpvt;
    return(0);
}

long DevMpf::read_write(void* v)
{
    dbCommon* pcommon = (dbCommon*)v;
    DevMpf* pdevMpf=(DevMpf*)pcommon->dpvt;
    if(pdevMpf) pdevMpf->lock();
    if(pdevMpf==0 || pdevMpf->connectState!=connectYes) {
	if(pdevMpf->connectState==connectQueueFull)
	    pdevMpf->connectState = connectYes;
	recGblSetSevr(pcommon,COMM_ALARM,INVALID_ALARM);
	if(pdevMpf->replyMessage) {
	    delete pdevMpf->replyMessage;
	    pdevMpf->replyMessage = 0;
	}
        if(pdevMpf) pdevMpf->unlock();
	return(MPF_NoConvert);
    }
    long rc = 0;
    if(pdevMpf->replyMessage!=0) {
	rc=pdevMpf->completeIO(pcommon,pdevMpf->replyMessage);
	pdevMpf->replyMessage=0; // allow next message to be handled
    } else {
	rc=pdevMpf->startIO(pcommon);
    }	
    pdevMpf->unlock();
    return rc;
}

void DevMpf::messageClientCallback(Message *message,void *clientPvt)
{
    dbCommon *pcommon = (dbCommon *)clientPvt;
    DevMpf *pdevMpf = (DevMpf *)pcommon->dpvt;
    pdevMpf->lock();
    if(message->getType()==messageTypeConnect) {
	ConnectMessage *pConnectMessage = (ConnectMessage *)message;
	pdevMpf->connectState = pConnectMessage->status;
	pdevMpf->connectIO(pcommon,message);
    } else if(message->getType()==messageTypeOutOfBand) {
        pdevMpf->outOfBandIO(pcommon,message);
    } else {
        if(pdevMpf->state==waitStateReply) {
            pdevMpf->state = waitStateNone;
            pdevMpf->sendWatchdog.cancel();
            epicsEventSignal(pdevMpf->sendWaitSem);
        }
        replyType type = (replyType)message->getClientType();
        switch(type) {
        case replyTypeNone:
            pdevMpf->outOfBandIO(pcommon,message);
            break;
        case replyTypeCompleteIO:
            if(!interruptAccept) {
                errlogPrintf("%s replyTypeCompleteIO but !interruptAccept\n",
                    pcommon->name);
		delete message;
            } else if(pdevMpf->replyMessage) {
                errlogPrintf("%s DevMpf new message before previous processed\n",
                    pcommon->name);
            } else {
                pdevMpf->replyMessage = message;
                callbackRequest(&pdevMpf->cb);
            }
            break;
        case replyTypeReceiveReply:
            pdevMpf->receiveReply(pcommon,message);
            break;
        default:
            errlogPrintf("%s DevMpf illegal replyType\n",pcommon->name);
        }
    }
    pdevMpf->unlock();
}
