// SerialConfigMessage.cc
/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPF is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <epicsThread.h>

#include "Message.h"
#include "SerialConfigMessage.h"

DLList* SerialConfigMessage::pFreeList = 0;

SerialConfigMessage:: SerialConfigMessage()
: Message(messageTypeSerialConfig),
    baud(0), stopBits(0), bitsPerChar(0), parity(0), flowControl(0)
{}

SerialConfigMessage:: SerialConfigMessage(messageType type)
: Message(type),
    baud(0), stopBits(0), bitsPerChar(0), parity(0), flowControl(0)
{}

int32 SerialConfigMessage:: toBuffer(char **buffer) const
{
    int32 nbytes = 0;

    nbytes = Message::toBuffer(buffer);
    nbytes += MPF::toBuffer(buffer,baud);
    nbytes += MPF::toBuffer(buffer,stopBits);
    nbytes += MPF::toBuffer(buffer,bitsPerChar);
    nbytes += MPF::toBuffer(buffer,parity);
    nbytes += MPF::toBuffer(buffer,flowControl);
    return(nbytes);
}

int32 SerialConfigMessage:: fromBuffer(const char **buffer)
{
    int32 nbytes=0;

    nbytes = Message::fromBuffer(buffer);
    nbytes += MPF::fromBuffer(buffer,&baud);
    nbytes += MPF::fromBuffer(buffer,&stopBits);
    nbytes += MPF::fromBuffer(buffer,&bitsPerChar);
    nbytes += MPF::fromBuffer(buffer,&parity);
    nbytes += MPF::fromBuffer(buffer,&flowControl);
    return(nbytes);
}

int32 SerialConfigMessage:: fromBufferSwitch(const char **buffer)
{
    int32 nbytes=0;

    nbytes = Message::fromBufferSwitch(buffer);
    nbytes += MPF::fromBufferSwitch(buffer,&baud);
    nbytes += MPF::fromBufferSwitch(buffer,&stopBits);
    nbytes += MPF::fromBufferSwitch(buffer,&bitsPerChar);
    nbytes += MPF::fromBufferSwitch(buffer,&parity);
    nbytes += MPF::fromBufferSwitch(buffer,&flowControl);
    return(nbytes);
}

void SerialConfigMessage:: print() const
{
    Message::print();
    printf("stopBits:%d ",(int)stopBits);
    printf("baud:%d ",baud);
    printf("bitsPerChar:%d ",(int)bitsPerChar);
    printf("parity:%d ",(int)parity);
    printf("flowControl:%d\n",(int)parity);
}

void *SerialConfigMessage:: operator new(size_t size)
{
    if(size!=sizeof(SerialConfigMessage)) {
        printf("SerialConfigMessage new size !=sizeof(SerialConfigMessage)\n");
        epicsThreadSuspendSelf();
    }
    return MPF::allocate(&pFreeList,size);
}

void SerialConfigMessage:: operator delete(void *p, size_t size)
{
    if(size==sizeof(SerialConfigMessage)) {
        MPF::free(pFreeList,p,size);
        return;
    }
    printf("SerialConfigMessage delete size !=sizeof(SerialConfigMessage)\n");
}
