// mpfRegister.cpp
/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPF is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include <iocsh.h>
#include <epicsExport.h>

#include "Message.h"
#include "MessagePvt.h"

#undef SUPPORT_REMOTE

extern "C" {

/*The following is allows a reference to it to bring in the rest
  of the registration code*/
void mpfRegistration() {}

// mrr
static const iocshFuncDef mrrFuncDef = {"mrr",0,0};
static void mrrCallFunc(const iocshArgBuf *)
{
    mrr();
}

//msr
static const iocshArg msrArg0 = { "location",iocshArgString};
static const iocshArg * const msrArgs[1] = {&msrArg0};
static const iocshFuncDef msrFuncDef = {"msr",1,msrArgs};
static void msrCallFunc(const iocshArgBuf *args)
{
    msr(args[0].sval);
}

// routerInit
static const iocshFuncDef routerInitFuncDef = {"routerInit",0,0};
static void routerInitCallFunc(const iocshArgBuf *)
{
    routerInit();
}

//localMessageRouterStart
static const iocshArg localMessageRouterStartArg0 = { "location",iocshArgInt};
static const iocshArg * const localMessageRouterStartArgs[1]
    = {&localMessageRouterStartArg0};
static const iocshFuncDef localMessageRouterStartFuncDef
    = {"localMessageRouterStart",1,localMessageRouterStartArgs};
static void localMessageRouterStartCallFunc(const iocshArgBuf *args)
{
    localMessageRouterStart(args[0].ival);
}

#ifdef SUPPORT_REMOTE
//tcpMessageRouterClientStart
static const iocshArg tcpMessageRouterClientStartArg0 =
    { "location",iocshArgInt};
static const iocshArg tcpMessageRouterClientStartArg1 =
    { "port",iocshArgInt};
static const iocshArg tcpMessageRouterClientStartArg2 =
    { "address",iocshArgString};
static const iocshArg tcpMessageRouterClientStartArg3 =
    { "bufsize",iocshArgInt};
static const iocshArg tcpMessageRouterClientStartArg4 =
    { "queueSize",iocshArgInt};
static const iocshArg * const tcpMessageRouterClientStartArgs[5]
    = {&tcpMessageRouterClientStartArg0,&tcpMessageRouterClientStartArg1,
       &tcpMessageRouterClientStartArg2,&tcpMessageRouterClientStartArg3,
       &tcpMessageRouterClientStartArg4};
static const iocshFuncDef tcpMessageRouterClientStartFuncDef
    = {"tcpMessageRouterClientStart",5,tcpMessageRouterClientStartArgs};
static void tcpMessageRouterClientStartCallFunc(const iocshArgBuf *args)
{
    tcpMessageRouterClientStart(args[0].ival,args[1].ival,
        args[2].sval,args[3].ival,args[4].ival);
}

//tcpMessageRouterServerStart
static const iocshArg tcpMessageRouterServerStartArg0 =
    { "location",iocshArgInt};
static const iocshArg tcpMessageRouterServerStartArg1 =
    { "port",iocshArgInt};
static const iocshArg tcpMessageRouterServerStartArg2 =
    { "address",iocshArgString};
static const iocshArg tcpMessageRouterServerStartArg3 =
    { "bufsize",iocshArgInt};
static const iocshArg tcpMessageRouterServerStartArg4 =
    { "queueSize",iocshArgInt};
static const iocshArg * const tcpMessageRouterServerStartArgs[5]
    = {&tcpMessageRouterServerStartArg0,&tcpMessageRouterServerStartArg1,
       &tcpMessageRouterServerStartArg2,&tcpMessageRouterServerStartArg3,
       &tcpMessageRouterServerStartArg4};
static const iocshFuncDef tcpMessageRouterServerStartFuncDef
    = {"tcpMessageRouterServerStart",5,tcpMessageRouterServerStartArgs};
static void tcpMessageRouterServerStartCallFunc(const iocshArgBuf *args)
{
    tcpMessageRouterServerStart(args[0].ival,args[1].ival,
        args[2].sval,args[3].ival,args[4].ival);
}
#endif /*SUPPORT_REMOTE*/

static void mpfRegister(void)
{
    iocshRegister(&mrrFuncDef,mrrCallFunc);
    iocshRegister(&msrFuncDef,msrCallFunc);
    iocshRegister(&routerInitFuncDef,routerInitCallFunc);
    iocshRegister(&localMessageRouterStartFuncDef,
        localMessageRouterStartCallFunc);
#ifdef SUPPORT_REMOTE
    iocshRegister(&tcpMessageRouterClientStartFuncDef,
        tcpMessageRouterClientStartCallFunc);
    iocshRegister(&tcpMessageRouterServerStartFuncDef,
        tcpMessageRouterServerStartCallFunc);
#endif /*SUPPORT_REMOTE*/
}


static void mpfRegistrar(void)
{
    mpfRegister();
}
epicsExportRegistrar(mpfRegistrar);
} // extern "C"
