// messageTest.cpp
/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPF is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include <epicsTime.h>
#include <epicsThread.h>
#include <iocsh.h>
#include <epicsExport.h>

#include "Message.h"
#include "ConnectMessage.h"
#include "Int32Message.h"
#include "DLList.h"
#include "test.h"

#define nmessages 10

typedef struct args {
    int ntimes;
}args;

static void messageTest(void *inargs)
{
    args *pargs = (args *)inargs;
    int ntimes = pargs->ntimes;
    Message 	*message;
    epicsTimeStamp    startTime;
    epicsTimeStamp    endTime;
    double	rate,seconds;
    DLList      *list;
    DLNode      *node;
    
    list = new DLList;
    epicsTimeGetCurrent(&startTime);
    for(int i=0; i<ntimes; i++) {
	for(int j=0; j<nmessages; j++) {
	    message = new Int32Message;
	    node = DLList::allocNode();
	    node->obj = (void *)message;
	    list->addEnd(node);
	}
	while((node = list->getFirst())) {
	    message = (Message *)node->obj;
	    list->freeNode(node);
	    delete message;
	}
    }
    epicsTimeGetCurrent(&endTime);
    seconds = epicsTimeDiffInSeconds(&endTime,&startTime);
    rate = ((double)ntimes * (double)nmessages)/seconds;
    printf("rate = %6.0f seconds=%6.0f\n", rate,seconds);
    return;
}

extern "C" {
int initMessageTest(int ntimes)
{
    args *pargs = (args *)calloc(1,sizeof(args));
    epicsThreadId threadId;

    pargs->ntimes = ntimes;
    threadId = epicsThreadCreate("messageTest",epicsThreadPriorityMedium,
        epicsThreadGetStackSize(epicsThreadStackMedium),
        (EPICSTHREADFUNC)messageTest,(void *)pargs);
    if(!threadId) printf("initMessageTest epicsThreadCreate Failure\n");
    return(0);
}

static const iocshArg initMessageTestArg0 = { "ntimes",iocshArgInt};
static const iocshArg * const initMessageTestArgs[1]
    = {&initMessageTestArg0};
static const iocshFuncDef initMessageTestFuncDef
    = {"initMessageTest",1,initMessageTestArgs};
static void initMessageTestCallFunc(const iocshArgBuf *args)
{
    initMessageTest(args[0].ival);
}


static void initMessageTestRegister(void)
{
    iocshRegister(&initMessageTestFuncDef,initMessageTestCallFunc);
}

static void initMessageTestRegistrar(void)
{
    initMessageTestRegister();
}

epicsExportRegistrar(initMessageTestRegistrar);

} // extern "C"
